<?php
if(!defined('_CLASS_MESSAGE')){
	define('_CLASS_MESSAGE', '_CLASS_MESSAGE');

require_once(INC_PATH.'/class.tnef.php');
require_once(INC_PATH.'/htmlfilter.php');
require_once(INC_PATH.'/punycode.php');
require_once(INC_PATH.'/certificate.php');
require_once(INC_PATH.'/class.msghandle.php');
require_once(INC_PATH.'/class.calendar.php');
require_once(INC_PATH.'/../ical/parser/autoload.php');

class Message extends MsgHandle {
	var $debug				= false;
	var $timezone			= '+0000';
	var $use_utf8			= false;
	var $temp_folder		= './';
	
	var $displayimages		= false;
	var $imageswidth		= 600;
	var $imagesheight		= 300;
	
	var $block_external_images	= false;
	var $allow_scripts		= false;
	var $current_level		= array();
	var $use_html			= true;
	var $only_body			= false;
	var $save_embed_attach	= false;
	var $decode_mstnef		= true;
	
	var $msg_sessid			= '';
	var $msg_userid			= '';
	var $msg_email			= '';
	var $msg_folder			= '';
	var $msg_id				= '';
	var $msg_content		= '';
	var $msg_header			= '';
	var $msg_body			= '';
	
	// private
	var $_content			= Array();

	function check_temp_folder() {
		$tempfolder = $this->temp_folder.'_attachments';
		if (!file_exists($tempfolder)) {
			mkdir($tempfolder);
			chmod($tempfolder, 0775);
		}
	}

	function add_body(&$strbody, &$ctype, $attach = false) {
		$charset = '';
		if ($ctype != ''){
			$charset = $this->get_charset($ctype);
		}

		if (strpos($strbody, "\0") !== false)
			$strbody = str_replace("\0", '', $strbody);
			
		if ($charset == '')
			$charset = $this->detect_charset($strbody);

		$bodycount = empty($this->_content['body']['count']) ? 0 : $this->_content['body']['count'];

		$bodyinfo = array();
		$bodyinfo['msgbody'] = $strbody;
		$bodyinfo['charset'] = $charset;
		$bodyinfo['attach'] = $attach;
		
		$this->_content['body'][$bodycount] = $bodyinfo;
		$this->_content['body']['count'] = $bodycount+1;

		unset($bodyinfo);
	}

	function decode_mime_string(&$mimestring) {
		$string = $mimestring;

		$pos = strpos($string, '=?');
		if ($pos !== false) {
			$arrTemp = array();
			$nCount = 0;
			do {
				$strtemp = substr($string, 0, $pos);
				if (trim($strtemp) != '') {
					$arrTemp[$nCount]['value'] = $strtemp;
					$arrTemp[$nCount]['charset'] = '';
					$nCount++;
				}
				
				$string = substr($string, $pos+2);
				$intpos = strpos($string, '?');
				$charset = strtolower(substr($string, 0, $intpos));
				$encode = strtolower(substr($string, $intpos+1, 1));
				$string = substr($string, $intpos+3);
				$endpos = strpos($string, '?=');
	
				if ($endpos === false){
					$mystring = $string;
					$string = '';
				}
				else{
					$mystring = substr($string, 0, $endpos);
					$string = substr($string, $endpos+2);
				}
	
				if ($encode == 'q') {
					if (strpos($mystring, '_'))
						$mystring = str_replace('_', ' ', $mystring);
					if (strpos($mystring, '=FFffff'))
						$mystring = str_replace('=FFffff', '=', $mystring); 
					if (strpos($mystring, '=00'))
						$mystring = str_replace('=00', '', $mystring); 
					
					$mystring = quoted_printable_decode($mystring); 
				}
				else if ($encode == 'b'){
					$mystring = base64_decode($mystring);
				}
				
				$arrTemp[$nCount]['value'] = $mystring;
				$arrTemp[$nCount]['charset'] = $charset;
				$nCount++;
	
				$pos = strpos($string, '=?');
			} while($pos !== false);
			
			if (trim($string) != '') {
				$arrTemp[$nCount]['value'] = $string;
				$arrTemp[$nCount]['charset'] = '';
				$nCount++;
			}
	
			if ($nCount > 1) {
				$samecharset = true;
				$charset = $arrTemp[0]['charset'];
				$result = '';
				foreach ($arrTemp as &$temp) {
					if (strcasecmp($charset, $temp['charset']) != 0) {
						$samecharset = false;
						break;
					}
					
					$result .= $temp['value'];
				}
				
				if (!$samecharset) {
					$charset = 'utf-8';
					$result = '';
					foreach ($arrTemp as &$temp) {
						if (strcasecmp($charset, $temp['charset']) == 0) {
							$result .= $temp['value'];
						}
						else {
							$tempcharset = $temp['charset'];
							
							if ($tempcharset == '')
								$tempcharset = $this->detect_charset($temp['value']);
								
							if (strcasecmp($charset, $tempcharset) == 0)
								$result .= $temp['value'];
							else
								$result .= convert($temp['value'], true, $tempcharset);
						}
					}
				}
			}
			else {
				$result = $arrTemp[0]['value'];
				$charset = $arrTemp[0]['charset'];
			}
	
//			if (preg_match('/koi8/i', $mimestring)) 
//				$result = convert_cyr_string($result, 'k', 'w');
		}
		else{
			$result = $string;

			if ($charset == '')
				$charset = $this->detect_charset($result);
		}
		
		unset($string);

		if (strpos($result, "\0"))
			$result = str_replace("\0", '', $result);

		$arrResult['value'] = trim($result);
		$arrResult['charset'] = $charset;

		return $arrResult;
	}

	function decode_header(&$header, $tolower = true) {
		$arrHeader = explode("\r\n", $header);

		$errorHeader = false;
		$lastHeaderName = '';
		$arrDecodedHeader = Array();
		
		foreach($arrHeader as &$strHeader) {
			if (empty($strHeader)) 
				continue;

			if (!preg_match('/^[0-9a-z_-]+:/i', $strHeader)){
				if ($strHeader[0] != ' ' && $strHeader[0] != "\t") {
					$errorHeader = true;
					continue;
				}
				
				if (!$errorHeader)
					$arrDecodedHeader[$lastHeaderName] .= ' '.trim($strHeader);
			}
			else {
				$iSplitPos = strpos($strHeader, ':');
				
				if ($iSplitPos !== false){
					$headerName = trim(substr($strHeader, 0, $iSplitPos));
					$headerValue = trim(substr($strHeader, $iSplitPos+1));
					
					if ($tolower)
						$headerName = strtolower($headerName);

					if ($arrDecodedHeader[$headerName] != '') 
						$arrDecodedHeader[$headerName] .= "\r\n".$headerValue;
					else 
						$arrDecodedHeader[$headerName] = $headerValue;
						
					$lastHeaderName = $headerName;
					$errorHeader = false;
				}
				else{
					if (!$errorHeader)
						$arrDecodedHeader[$headerName] .= ' '.trim($strHeader);
				}
			}
		}
		
		unset($arrHeader);

		return $arrDecodedHeader;
	}

	function parse_addresses(&$mail, $trans = 0, $utf8 = false) {
		$arrAddr = Array();

		$strmail = preg_replace("/(\t|\r|\n)/", '', $mail);
		if (trim($strmail) == '') 
			return $arrAddr;

		$counter = 0;  
		$inthechar = 0;
		$chartosplit = ',;'; 
		$protectchar = '"'; 
		$temp = '';
		$lt = '<'; 
		$gt = '>';
		$closed = 1;
		$prevchar = '';

		$iLen = strlen($strmail);
		for($i = 0; $i < $iLen; $i++) {
			$thischar = $strmail[$i];
			
			if ($thischar == $lt && $closed)
				$closed = 0;
				 
			if ($thischar == $gt && !$closed) 
				$closed = 1;
				
			if ($thischar == $protectchar && $prevchar != '\\') 
				$inthechar = ($inthechar) ? 0 : 1;
				
			if (strpos($chartosplit, $thischar) !== false) {
				if (!$inthechar && $closed) {
					if (trim($temp) != '')
						$arrAddr[] = $temp; 
						
					$temp = '';
					$prevchar = '';
				}
				else if ($inthechar && $prevchar == $gt) {
					if (trim($temp) != '')
						$arrAddr[] = $temp; 
						
					$temp = '';
					$prevchar = '';
				}
				else {
					$temp .= $thischar;
					$prevchar = $thischar;
				}
			} 
			else {
				$temp .= $thischar;
				$prevchar = $thischar;
			}			
		}

		if (trim($temp) != '')
			$arrAddr[] = trim($temp);

		foreach($arrAddr as &$address) {
			$thisPart = trim(preg_replace('/^"(.*)"$/i', '\\1', trim($address)));
			
			if ($thisPart != '') {
				if (preg_match('/(.*)<(.*)>/i', $thisPart, $regs)) {
					$name = trim($regs[1]);
					$email = trim($regs[2]);
				} 
				else if (preg_match('/(\'|\")(.*)(\'|\")(.*)/i', $thisPart, $regs)) {
					$name = trim($regs[2]);
					$email = trim($regs[4]);
				} 
				else if (($iPos = strpos($thisPart, ' ')) !== false) {
					$name = trim(substr($thisPart, 0, $iPos));
					$email = trim(substr($thisPart, $iPos+1));
				}
				else if (($iPos = strpos($thisPart, '@')) !== false){
					$name = '';
					$email = $thisPart;
				}
				else {
					$name = $thisPart;
					$email = '';
				}

				$email = preg_replace('/<(.*)\\>/', '\\1', $email);
				$name = preg_replace('/"(.*)"/', '\\1', trim($name));
				$name = preg_replace('/[[:punct:]]\((.*)[[:punct:]]\)/', '(\\1)', $name);

				if ($name != '' || $email != ''){
					$addrItem = array();
					$addrItem['name'] = $this->decode_mime_string($name);
					
					switch($trans){
					default:
					case 0:
						$addrItem['mail'] = $email;
						break;
					case 1:
						$addrItem['mail'] = decode_punycode_mail($email, $utf8);
						break;
					case 2:
						$addrItem['mail'] = encode_punycode_mail($email, $utf8);
						break;
					}
					
					$address = $addrItem;
					
					unset($addrItem);
				}
				
				unset($name);
				unset($email);
			}
		}
		
		return $arrAddr;
	}

	function build_alternative_body(&$body, $ctype) {
		$boundary = $this->get_boundary($ctype);
		$parts = $this->split_parts($boundary, $body);

		foreach($parts as &$partbody) {
			$this->fetch_structure($partbody, $email);
			$partbody = null;
			
			$headers = $this->decode_header($email['header']);
			
			$ctype = explode(';', $headers['content-type'], 2); 
			$type = strtolower($ctype[0]);
			
			$types = explode('/', $type);
			$maintype = trim(strtolower($types[0]));
			$subtype = trim(strtolower($types[1]));

			if ($maintype == 'multipart') {
				$this->process_message($email['header'], $email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
			}
			else if ($maintype == 'text') {
				if ($subtype == 'plain') {
					$plainpart = $email;
					$plainpart['headers'] = $headers;
					$plainpart['type'] = $type;
				}
				else if ($subtype == 'html') {
					$htmlpart = $email;
					$htmlpart['headers'] = $headers;
					$htmlpart['type'] = $type;
				}
				else if ($subtype == 'calendar') {
					$this->process_message($email['header'], $email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
				}
				else {
					$this->build_attach($email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
				}
			}
			else{
				$this->build_attach($email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
			}
			
			unset($email);
		}

		if ($this->use_html)
			$part = (isset($htmlpart)) ? $htmlpart : $plainpart;
		else
			$part = (isset($plainpart)) ? $plainpart : $htmlpart;

		if (isset($part)){
			$strbody = $this->compile_body($part['body'], $part['headers']['content-type'], $part['headers']['content-transfer-encoding']);

			if (!$this->use_html){
				if ($part['type'] == 'text/html') 
					$strbody = $this->html2text($strbody);

				$this->build_text_body($strbody);
			}
			else {
				if ($part['type'] == 'text/plain') 
					$this->build_text_body($strbody);
			}

			$this->add_body($strbody, $part['headers']['content-type']);
			
			unset($strbody);
			unset($part);
		}
		
		unset($parts);
	}

	function build_complex_body(&$body, $ctype) {
		$rtype = trim(substr($ctype, strpos($ctype, 'type=')+5, strlen($ctype)));

		if (strpos($rtype, ';') != 0)
			$rtype = substr($rtype, 0, strpos($rtype,';'));
		if (substr($rtype, 0, 1) == '"' && substr($rtype, -1) == '"')
			$rtype = substr($rtype, 1, strlen($rtype)-2);

		$boundary = $this->get_boundary($ctype);
		$parts = $this->split_parts($boundary, $body);

		foreach($parts as &$partbody) {
			$this->fetch_structure($partbody, $email);
			$partbody = null;
			
			$headers = $this->decode_header($email['header']);
			
			$arrtype = explode(';', $headers['content-type'], 2);
			$rctype = trim(strtolower($arrtype[0]));
			
			$namematch = preg_match('/name=/i', $headers['content-disposition'].$headers['content-type']);
			$is_download = ($namematch != 0) || ($headers['content-id'] != '');
			
			if ($is_download && $namematch == 0 && ($rctype == 'text/plain' || $rctype == 'text/html'))
				$is_download = false;

			if ($email['header'] == '')
				$rctype = 'text/plain';
				
			if ($rctype == 'multipart/related'
				|| $rctype == 'multipart/mixed'){
				$this->build_complex_body($email['body'], $headers['content-type']);
			}
			elseif ($rctype == 'multipart/alternative'
				|| $rctype == 'multipart/appledouble') {
				$this->build_alternative_body($email['body'], $headers['content-type']);
			} 
			elseif ($rctype == 'application/ms-tnef') {
				$strbody = $this->compile_body($email['body'], $headers['content-type'], $headers['content-transfer-encoding']);
				$this->extract_tnef($strbody, $boundary);
			}
			else if ($rctype == 'text/calendar') {
				$this->process_message($email['header'], $email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
			}
			else if (!$is_download) {
				if ($rctype == 'text/plain') {
					$strbody = $this->compile_body($email['body'], $headers['content-type'], $headers['content-transfer-encoding']);
					$this->build_text_body($strbody);
	
					$this->add_body($strbody, $headers['content-type']);
				} 
				else if ($rctype == 'text/html') {
					$strbody = $this->compile_body($email['body'], $headers['content-type'], $headers['content-transfer-encoding']);

					if (!$this->use_html) {
						$strbody = $this->html2text($strbody);
						$this->build_text_body($strbody);
					}
	
					$this->add_body($strbody, $headers['content-type']);
				}
				else{
					$this->process_message($email['header'], $email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
				}
			}
			else {
				$thisattach = $this->build_attach($email['body'], $headers['content-type'], $headers['content-transfer-encoding'], $headers['content-id'], $headers['content-disposition']);
			} 
			
			unset($email);
			unset($headers);
			unset($strbody);
		}
		
		unset($parts);
	}

	function build_smimecrypt_body(&$header, &$body){
		$bRet = false;
		if (read_my_acitve_cert($cert, $key)){
			$fileid = md5(uniqid(microtime(), true));
			$encryptfile = $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_e_'.$fileid;
			$decryptfile = $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_d_'.$fileid;

			$msgcontent = $header."\r\n\r\n".$body;
			if (file_put_contents($encryptfile, $msgcontent)){
				if (openssl_pkcs7_decrypt($encryptfile, $decryptfile, $cert, $key)){
					$msgcontent = $this->get_mail_content($decryptfile);
					$this->remove_crlfdot($msgcontent);		

					$this->fetch_structure($msgcontent, $email);
					unset($msgcontent);
				
					$this->_content['header'] = $email['header'];
					$this->_content['body']['count'] = 0;
					
					$mailinfo = $this->get_mail_info($email['header']);
					if (isset($mailinfo['date']))
						$this->_content['date'] = $mailinfo['date'];
					if (isset($mailinfo['subject']))
						$this->_content['subject'] = $mailinfo['subject'];
					if (isset($mailinfo['message-id']))
						$this->_content['message-id'] = $mailinfo['message-id'];
					if (isset($mailinfo['from']))
						$this->_content['from'] = $mailinfo['from'];
					if (isset($mailinfo['to']))
						$this->_content['to'] = $mailinfo['to'];
					if (isset($mailinfo['cc']))
						$this->_content['cc'] = $mailinfo['cc'];
					if (isset($mailinfo['reply-to']))
						$this->_content['reply-to'] = $mailinfo['reply-to'];
					if (isset($mailinfo['priority']))
						$this->_content['priority'] = $mailinfo['priority'];
					if (isset($mailinfo['disposition-notification-to']))
						$this->_content['disposition-notification-to'] = $mailinfo['disposition-notification-to'];
					if (isset($mailinfo['content-type']))
						$this->_content['content-type'] .= $mailinfo['content-type'];
					if (isset($mailinfo['content-transfer-encoding']))
						$this->_content['content-transfer-encoding'] = $mailinfo['content-transfer-encoding'];

					$this->process_message($email['header'], $email['body'], $mailinfo['content-type'], $mailinfo['content-transfer-encoding'], $mailinfo['content-id'], $mailinfo['content-disposition']);

					unset($email);
					unset($mailinfo);
					
					$bRet = true;
				}
			}
			unset($msgcontent);

			@unlink($encryptfile);
			@unlink($decryptfile);			
		}
		
		return $bRet;
	}

	function build_smimesigned_body(&$header, &$body){
		$fileid = md5(uniqid(microtime(), true));
		$signedfile = $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_s_'.$fileid;
		$certfile = $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_c_'.$fileid;

		$bRet = false;
		$msgcontent = $header."\r\n\r\n".$body;
		if (file_put_contents($signedfile, $msgcontent)){
			if (openssl_pkcs7_verify($signedfile, PKCS7_DETACHED|PKCS7_NOVERIFY, $certfile)){
				$arrTemp = Array();
			
				$arrTemp['index']				= is_array($this->_content['attachments']) ? count($this->_content['attachments']) : 0;
				$arrTemp['name']['value'] 		= 'certificate.crt';
				$arrTemp['name']['charset'] 	= '';
				$arrTemp['size'] 				= filesize($certfile);
				$arrTemp['content-type'] 		= 'application/x-x509-ca-cert';
				$arrTemp['content-disposition'] = 'attachment';
				$arrTemp['filename'] 			= $certfile;
				$arrTemp['embed'] 				= true;
				
				$this->_content['attachments'][] 	= $arrTemp;
				
				unset($arrTemp);
				
				$bRet = true;
			}
		}
		unset($msgcontent);
		
		@unlink($signedfile);
		
		return $bRet;
	}
	
	function htmlspecialchars_jis(&$text) {
		if (empty($text)) 
			return $text;
		
		$esc = chr(27);
		if (strpos($text, $esc.'$B') === false)
			return htmlspecialchars($text, ENT_COMPAT, 'ISO-8859-1');
			
		$strtext = $esc.'$B'.$esc.'$B'.$text;
		$strtext = str_replace($esc.'(B', $esc.'$B', $strtext);
		$trans = explode($esc.'$B', $strtext);
		$enc = 0;
		
		$ret = '';
		foreach($trans as &$val) {
			if ($enc == 0) {
				$val .= '';
				if ($val != '') 
		       		$ret .= htmlspecialchars($val, ENT_COMPAT, 'ISO-8859-1');
		       		
		       	$enc = 1;
		   	} 
		   	else {
				$val .= '';
				if ($val != '') 
					$ret .= $esc.'$B'.$val.$esc.'(B';
					
				$enc = 0;
			}
		}
		
		unset($strtext);
		unset($trans);
		
		return $ret;
	}

 	function build_text_body(&$body) {
		if (!$this->only_body) {
			$body = $this->htmlspecialchars_jis($body);
			$body = $this->make_link_clickable($body);
	
//			$body = preg_replace("/(\r\n|\n|\r)/", '<br/>\\1', $body);
			$body = "<pre style=\"white-space:pre-wrap; white-space:-moz-pre-wrap; white-space:-pre-wrap; white-space:-o-pre-wrap; word-wrap:break-word;\">\r\n".$body."\r\n</pre>";
		}
	}

	function process_message(&$header, &$body, $ctype, $ctenc, $cid, $cdisp) {
		if ($ctype == '') 
			$ctype = 'text/plain';

		$type = $ctype;
		$subctype = '';
		$iPos = strpos($ctype, ';');
		if ($iPos !== false) {
			$type = substr($ctype, 0, $iPos);
			$subctype = trim(substr($ctype, $iPos+1));
		}
		$types = explode('/', $type);

		$maintype = trim(strtolower($types[0]));
		$subtype = trim(strtolower($types[1]));

		$pos = strpos($cdisp, ';');
		if ($pos === false)
			$subcdisp = trim($cdisp);
		else
			$subcdisp = trim(substr($cdisp, 0, $pos));

		if ($subcdisp == 'attachment')
			$maintype = 'multipart';

		switch($maintype) {
		case 'text':
			$msgbody = $this->compile_body($body, $ctype, $ctenc);

			switch($subtype) {
			case 'html':
				if (!$this->use_html) {
					$msgbody = $this->html2text($msgbody);
					$this->build_text_body($msgbody);
				}
				break;
			case 'calendar':
				$this->build_attach($body, $ctype, $ctenc, $cid, $cdisp);
				
				global $selected_common_language;
				getlanguagetext($selected_common_language, 'Calendar');
				
				$charset = $this->get_charset($ctype);

				$ical = new \ICS\Parser\ICal($msgbody, array('skipRecurrence' => true));
				$ical->charset = $charset;

				if ($ical->hasEvents()) {
					$msgbody = '<style>'."\r\n";
					$msgbody .= 'table {border: 1px solid #ccc; min-width:400px; margin-bottom:10px; width:100%}'."\r\n";
					$msgbody .= 'td {border-bottom: 1px solid #ccc; padding: 3px}'."\r\n";
					$msgbody .= 'td:first-child {background-color:#aaa; color:#fff; padding-right:20px; width:80px}'."\r\n";
					$msgbody .= '</style>'."\r\n";

					foreach ($ical->cal['VEVENT'] as &$evt) {
					   	if ($evt['DTSTART_array'][3]) {
						    $dtstart = $ical->iCalDateToDateTime($evt['DTSTART_array'][3], false);
						    $starttime = $dtstart->getTimestamp();
						}
						else {
							$starttime = strtotime($evt['DTSTART']);
						}
						
					   	if ($evt['DTEND_array'][3]) {
						    $dtend = $ical->iCalDateToDateTime($evt['DTEND_array'][3], false);
						    $endtime = $dtend->getTimestamp();
						}
						else {
							$endtime = strtotime($evt['DTEND']);
						}
											
						$msgbody .= '<table>'."\r\n";					
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_title'].'</td><td>'.(($evt['SUMMARY'] != '') ? htmlspecialchars($evt['SUMMARY']) : '&nbsp;').'</td></tr>'."\r\n";
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_starttime'].'</td><td>'.convert(strftime($GLOBALS['date_time_format'], $starttime)).'</td></tr>'."\r\n";
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_endtime'].'</td><td>'.convert(strftime($GLOBALS['date_time_format'], $endtime)).'</td></tr>'."\r\n";
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_location'].'</td><td>'.(($evt['LOCATION'] != '') ? htmlspecialchars($evt['LOCATION']) : '&nbsp;').'</td></tr>'."\r\n";
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_organizer'].'</td><td>'.(($evt['ORGANIZER'] != '') ? htmlspecialchars($evt['ORGANIZER']) : '&nbsp;').'</td></tr>'."\r\n";
					    $msgbody .= '<tr><td>'.$GLOBALS['cal_attendee'].'</td><td>'.(($evt['ATTENDEE'] != '') ? htmlspecialchars($evt['ATTENDEE']) : '&nbsp;').'</td></tr>'."\r\n";
						$msgbody .= '<tr><td>'.$GLOBALS['cal_content'].'</td><td>';
					    if ($evt['DESCRIPTION'] != '')
						    $msgbody .= nl2br(htmlspecialchars($evt['DESCRIPTION']));
					    if ($evt['COMMENT'] != '')
						    $msgbody .= nl2br(htmlspecialchars($evt['COMMENT']));
						$msgbody .= '</td></tr>'."\r\n";
					    if ($evt['RRULE'] != '')
						    $msgbody .= '<tr><td>'.$GLOBALS['cal_repeat'].'</td><td>'.(($evt['RRULE'] != '') ? htmlspecialchars($evt['RRULE']) : '&nbsp;').'</td></tr>'."\r\n";
						$msgbody .= '</table>'."\r\n";
					}
				}
	
				break;
			default:
				$this->extract_uuencoded($msgbody);
				$this->build_text_body($msgbody);
				break;
			}

			$this->add_body($msgbody, $ctype);
			
			unset($msgbody);
			
			break;
		case 'multipart':
			if (preg_match('/'.$subtype.'/i', 'mixed,related,report,appledouble'))
				$subtype = 'complex';
			
			switch($subtype) {
			case 'alternative':
				$this->build_alternative_body($body, $subctype);
				break;
			case 'signed':
				$this->build_smimesigned_body($header, $body);
			case 'complex':
				$this->build_complex_body($body, $ctype);
				break;
			default:
				$this->build_attach($body, $ctype, $ctenc, $cid, $cdisp);
				break;
			}
			
			break;
		case 'application':
			switch($subtype) {
			case 'pkcs7-mime':
			case 'x-pkcs7-mime':
				if (stripos($subctype, 'smime-type=signed-data') != false) {
					if (stripos($subctype, 'name=smime.p7m') != false)
						$this->Decode($header."\r\n".$body);

					$this->build_smimesigned_body($header, $body);
				}
				else {
					if ($this->build_smimecrypt_body($header, $body))
						break;
				}
			default:
				$this->build_attach($body, $ctype, $ctenc, $cid, $cdisp);
				break;
			}
		
			break;
		default:
			$this->build_attach($body, $ctype, $ctenc, $cid, $cdisp);
			break;
		}
	}

	function build_attach(&$body, &$ctype, &$ctenc, &$cid, &$cdisp) {
		if ($this->only_body)
			return false;
			
		$pos = strpos($ctype, ';');
		if ($pos === false)
			$subctype = trim($ctype);
		else
			$subctype = trim(substr($ctype, 0, $pos));
		
		$subctype = strtolower($subctype);
		
		$pos = strpos($cdisp, ';');
		if ($pos === false)
			$subcdisp = trim($cdisp);
		else
			$subcdisp = trim(substr($cdisp, 0, $pos));

		$is_embebed = ((($cid != '') && (stristr($cdisp, 'inline') !== false))
						|| ($subctype == 'application/pkcs7-signature')		
						|| ($subctype == 'application/x-pkcs7-signature'));

		$strbody = $this->compile_body($body, $ctype, $ctenc);

		$filename = $this->get_filename($cdisp, $ctype);
		$index = is_array($this->_content['attachments']) ? count($this->_content['attachments']) : 0;
		$arrName = $this->decode_mime_string($filename);

		if ($arrName['value'] == '') {
			global $mimetypes;
			if ($subctype == 'message/rfc822') 
				$attext = '.eml';
			else if ($subctype == 'text/calendar') 
				$attext = '.ics';
			else if (($attext = array_search($subctype, $mimetypes)) === false)
				$attext = '.txt';
			
			$arrName['value'] = sprintf('attachment_%d%s', $index+1, $attext);
			$arrName['charset'] = 'iso-8859-1';
		}
		
		unset($headers);
		
		$arrTemp = array();
		
		$arrTemp['index'] = $index;
		$arrTemp['name'] = $arrName;
		$arrTemp['size'] = strlen($strbody);
		$arrTemp['charset'] = $this->get_charset($ctype);
		$arrTemp['content-type'] = $subctype;
		$arrTemp['content-disposition'] = $subcdisp;
		if ($cid != '')
			$arrTemp['content-id'] = $cid;
			
		$arrTemp['filename'] = $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_'.md5($this->msg_folder.'_'.$this->msg_id.'_'.join('|', array_merge($this->current_level, array($arrTemp['index']))));

		file_put_contents($arrTemp['filename'], $strbody);

		if ($is_embebed)
			$arrTemp['embed'] = $this->save_embed_attach ? false : true;
		else
			$arrTemp['embed'] = false;
		
		$this->_content['attachments'][] = $arrTemp;

		unset($strbody);
		
		return $arrTemp;
	}

	function compile_body($body, $ctype, $ctenc) {
		$ctenc = explode(' ', $ctenc); 
		$ctenc = $ctenc[0];
	
		$body = trim($body);
		
		if (strcasecmp($ctenc, 'base64') == 0){
			$len = strlen($body);
			while($len % 4) {
				$body .= '=';
				$len++;
			}

			$body = base64_decode($body);
		}
		else if (strcasecmp($ctenc, 'quoted-printable') == 0) {
//			$body = preg_replace("/=A0/i", '', $body);
			$body = quoted_printable_decode($body);
		}
		else if (strcasecmp($ctenc, 'uuencode') == 0) {
			$regex = "/(begin ([0-7]{3}) (.+))\r\n(.+)\r\nend/Us";
	        preg_match_all($regex, $body, $matches);

			$body = $this->uudecode($matches[4][0]);
		}

//		if (preg_match('/koi8-r/i', $ctype)) 
//			$body = convert_cyr_string($body, 'k', 'w');

		return $body;
	}

	function get_mail_info(&$header) {
		$myarray = array();
		
		$headers = $this->decode_header($header);

		unset($header);
		
		if (!empty($headers['content-type']))
			$myarray['content-type'] = $headers['content-type'];
		if (!empty($headers['content-transfer-encoding']))
			$myarray['content-transfer-encoding'] = str_replace('GM', '-', $headers['content-transfer-encoding']);
		if (!empty($headers['content-id']))
			$myarray['content-id'] = $headers['content-id'];
		if (!empty($headers['content-disposition']))
			$myarray['content-disposition'] = $headers['content-disposition'];
		if (!empty($headers['x-priority']))
			$myarray['priority'] = $headers['x-priority'][0];
		if (!empty($headers['message-id']))
			$myarray['message-id'] = trim($headers['message-id']);
		if (!empty($headers['in-reply-to']))
			$myarray['in-reply-to'] = trim($headers['in-reply-to']);
		if (!empty($headers['references']))
			$myarray['references'] = trim($headers['references']);
		if (!empty($headers['x-attachment']))
			$myarray['attach'] = $headers['x-attachment'];
		if (!empty($headers['x-has-attach']))
			$myarray['attach'] = $headers['x-has-attach'];
		if (!isset($myarray['attach'])){
			if (preg_match('/(multipart\/mixed)/i', $headers['content-type']))
				$myarray['attach'] = 'yes';
		}		
		if ($headers['sender'] != '') {
			$arrsend = explode("\r\n", $headers['sender']);
			$arrsend = $this->parse_addresses($arrsend[0], 1, $this->use_utf8);
			
			$sender = $arrsend[0]['mail'];
			if (strpos($sender, '+arep_=') !== false || strpos($sender, '+afwd_=') !== false
				|| (strpos($sender, 'auto_') !== false && strpos($sender, '+') !== false && strpos($sender, '=') !== false))
				$sender = '';
			$myarray['sender'] = $sender;
		}
		if ($headers['x-wm-sender'] != '') {
			$arrsend = explode("\r\n", $headers['x-wm-sender']);
			$arrsend = $this->parse_addresses($arrsend[0], 1, $this->use_utf8);
			
			$sender = $arrsend[0]['mail'];
			if (strpos($sender, '+arep_=') !== false || strpos($sender, '+afwd_=') !== false
				|| (strpos($sender, 'auto_') !== false && strpos($sender, '+') !== false && strpos($sender, '=') !== false))
				$sender = '';
			$myarray['wmsender'] = $sender;
		}
		if ($headers['x-wm-status'] != '')
			$myarray['wmstatus'] = trim($headers['x-wm-status']);
		if ($headers['x-wm-type'] != '')
			$myarray['wmtype'] = trim($headers['x-wm-type']);
		if ($headers['x-wm-schedule'] != '')
			$myarray['wmschedule'] = trim($headers['x-wm-schedule']);			
		if ($headers['x-wm-sendtime'] != '')
			$myarray['wmsendtime'] = trim($headers['x-wm-sendtime']);
		if ($headers['x-wm-secret'] != '')
			$myarray['wmsecret'] = trim($headers['x-wm-secret']);			

		if ($headers['x-qq-style'] == '800')
			$myarray['wmtype'] = 'encrypt';
			
		$received = chop($headers['received']);
		$user_date = chop($headers['date']);
		if (preg_match('/([0-9]{1,2}[ ]+[A-Z]{3}[ ]+[0-9]{4}[ ]+[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})[ ]?((\+|-)[0-9]{4})?/i', $received, $regs)) {
			//eg. Tue, 4 Sep 2001 16:22:31 -0000
			$mydate = $regs[1];
			$mytimezone = $regs[2];
			
			if (empty($mytimezone)){
				if (preg_match('/((\\+|-)[0-9]{4})/i', $user_date, $regs)) 
					$mytimezone = $regs[1];
				else 
					$mytimezone = $this->timezone;
			}
		}
		else if (preg_match('/(([A-Z]{3})[ ]+([0-9]{1,2})[ ]+([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})[ ]+([0-9]{4}))/i', $received, $regs)) {
			//eg. Tue Sep 4 16:26:17 2001 (Cubic Circle's style)
			$mydate = $regs[3].' '.$regs[2].' '.$regs[5].' '.$regs[4];
			
			if (preg_match('/((\\+|-)[0-9]{4})/i', $user_date, $regs)) 
				$mytimezone = $regs[1];
			else 
				$mytimezone = $this->timezone;
		} 
		else if (preg_match('/([0-9]{1,2}[ ]+[A-Z]{3}[ ]+[0-9]{4}[ ]+[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})[ ]?((\+|-)[0-9]{4})?/i', $user_date, $regs)) {
			//eg. Tue, 4 Sep 2001 16:22:31 -0000 (from Date header)
			$mydate = $regs[1];
			$mytimezone = $regs[2];
			if (empty($mytimezone)){
				if (preg_match('/((\\+|-)[0-9]{4})/i', $user_date, $regs)) 
					$mytimezone = $regs[1];
				else 
					$mytimezone = $this->timezone;
			}
		} 
		else {
			$mydate = date('d M Y H:i');
			$mytimezone	= $this->timezone;
		}
		
		if (!empty($headers['subject']))
			$myarray['subject'] = $this->decode_mime_string($headers['subject']);

		if (!empty($mydate))
			$myarray['date'] = $this->build_mime_date($mydate, $mytimezone);
		if (!empty($headers['from']))
			$myarray['from'] = $this->parse_addresses($headers['from'], 1, $this->use_utf8);
		if (!empty($headers['to']))
			$myarray['to'] = $this->parse_addresses($headers['to'], 1, $this->use_utf8);
		if (!empty($headers['cc']))
			$myarray['cc'] = $this->parse_addresses($headers['cc'], 1, $this->use_utf8);
		if (!empty($headers['reply-to']))
			$myarray['reply-to'] = $this->parse_addresses($headers['reply-to'], 1, $this->use_utf8);
		if (!empty($headers['disposition-notification-to']))
			$myarray['disposition-notification-to'] = $this->parse_addresses($headers['disposition-notification-to'], 1, $this->use_utf8);
		
		unset($headers);
		
		return $myarray;
	}

	function build_mime_date(&$mydate, $timezone = '+0000') {
		if (!preg_match('/((\\+|-)[0-9]{4})/i', $timezone)) 
			$timezone = '+0000';
	
		if (($intdate = @strtotime($mydate)) == -1)
			return time(); 
			
		if (preg_match('/(\\+|-)+([0-9]{2})([0-9]{2})/i', $timezone, $regs)) 
			$datetimezone = (intval($regs[1].$regs[2])*3600)+(intval($regs[1].$regs[3])*60);
		else 
			$datetimezone = 0;

		if (preg_match('/(\\+|-)+([0-9]{2})([0-9]{2})/i', $this->timezone, $regs))
			$usertimezone = (intval($regs[1].$regs[2])*3600)+(intval($regs[1].$regs[3])*60);
		else 
			$usertimezone = 0;
		
		if ($datetimezone != $usertimezone)	
			$diff = $usertimezone - $datetimezone;
		else
			$diff = 0;

		return ($intdate+$diff);
	}
	
	function replace_embed_image() {
		foreach ($this->_content['attachments'] as &$attach) {
			$attachtag = join('|', array_merge($this->current_level, array($attach['index'])));
			$viewlink = 'main.php?sessid='.$this->msg_sessid.'&act=download&folder='.urlencode($this->msg_folder).'&msgid='.$this->msg_id.'&attach='.$attachtag.'&opt=onlineview&retid='.time();
			
			if ($attach['content-id'] != '') {
				$embedid = preg_replace('/<(.*)\\>/', '\\1', $attach['content-id']);
				if ($embedid != '')
					$embedid = 'cid:'.trim($embedid);	
			}
			else if ($attach['content-location'] != '') {
				$embedid = trim($attach['content-location']);	
			}

			$inbody = false;
			if ($embedid != '') {
				$len = strlen($embedid);
				$newlen = strlen($thisfile);
				for ($m = 0; $m < $this->_content['body']['count']; $m++){
					$msgbody = &$this->_content['body'][$m]['msgbody'];

					$pos = 0;
					while(1) {
						$pos = strpos($msgbody, $embedid, $pos);
						if ($pos === false) 
							break;

						$msgbody = substr_replace($msgbody, $viewlink, $pos, $len);					
						$pos += $newlen;
						
						$inbody = true;
					}
				}
				
				if ($inbody) {
					$attach['viewlink'] = $viewlink;
					$attach['embed'] = true;
				}
			}
			
			if (!$inbody && $this->displayimages) {
				$ext = strtolower(strrchr($attach['name']['value'], '.'));
				$allowed_ext = Array('.gif', '.jpg', '.jpeg', '.png', '.jpe');

				if (in_array($ext, $allowed_ext)) {
					$imgInfo = getimagesize($attach['filename']);
					$width = $imgInfo[0];
					$height = $imgInfo[1];
					if ($imgInfo[0] > $this->imageswidth)
						$rate = floatval($this->imageswidth/$width);
					else if ($imgInfo[1] > $this->imagesheight)
						$rate = floatval($this->imagesheight/$height);
					else
						$rate = 1;
						
					$width = ceil($rate*$width);
					$height = ceil($rate*$height);
					
					$strbody = '<a href="'.$viewlink.'" target=_blank><img src="'.$viewlink.'" border=0 width="'.$width.'" height="'.$height.'"></a>';
					$strctype = '';
					$this->add_body($strbody, $strctype, true);
				}
			}
		}
	}

	function Decode($msg = '') {
		if ($msg != '')
			$this->msg_content = $msg;

		if ($this->msg_content == '')
			return array();
			
		$this->remove_crlfdot($this->msg_content);
		$this->fetch_structure($this->msg_content, $email);

		$this->msg_content = null;

		$this->_content = array();

		$this->_content['header'] = $email['header'];
		$this->_content['body']['count'] = 0;
		
		$mailinfo = $this->get_mail_info($email['header']);

		$this->_content['date'] = $mailinfo['date'];
		$this->_content['subject'] = $mailinfo['subject'];
		$this->_content['from'] = $mailinfo['from'];
		$this->_content['to'] = $mailinfo['to'];
		$this->_content['cc'] = $mailinfo['cc'];
		$this->_content['reply-to'] = $mailinfo['reply-to'];
		$this->_content['priority'] = $mailinfo['priority'];
		$this->_content['attach'] = $mailinfo['attach'];
		$this->_content['content-type'] = $mailinfo['content-type'];
		$this->_content['disposition-notification-to'] = $mailinfo['disposition-notification-to'];
		$this->_content['content-transfer-encoding'] = $mailinfo['content-transfer-encoding'];
		$this->_content['message-id'] = $mailinfo['message-id'];
		$this->_content['in-reply-to'] = $mailinfo['in-reply-to'];
		$this->_content['references'] = $mailinfo['references'];
		$this->_content['sender'] = $mailinfo['sender'];
		$this->_content['wmsender'] = $mailinfo['wmsender'];
		$this->_content['wmstatus'] = $mailinfo['wmstatus'];
		$this->_content['wmtype'] = $mailinfo['wmtype'];
		$this->_content['wmschedule'] = $mailinfo['wmschedule'];
		$this->_content['wmsendtime'] = $mailinfo['wmsendtime'];
		$this->_content['wmsecret'] = $mailinfo['wmsecret'];

		$this->process_message($email['header'], $email['body'], $mailinfo['content-type'], $mailinfo['content-transfer-encoding'], $mailinfo['content-id'], $mailinfo['content-disposition']);

		unset($email);
		
		$this->replace_embed_image();
		
		return $this->_content;
	}

	function split_parts(&$boundary, &$body) {
		if ($boundary == '') {
			$arrRet[0] = $body;
			return $arrRet;
		}
			
		$beginPos = strpos($body, $boundary);
		if ($beginPos !== false){
			$startPos = $beginPos + strlen($boundary);
			
			$endboundary = $boundary.'--';
			$endPos = strpos($body, $endboundary, $startPos);
			if ($endPos === false)
				$endPos = strlen($body);
	
			$lenbody = $endPos - $startPos;
			$newbody = substr($body, $startPos, $lenbody);
			$newbody = trim($newbody, "\r\n");
			
			$arrRet = explode($boundary, $newbody);
			
			unset($newbody);
		}
		else {
			$arrRet = explode($boundary, $body);
		}		
		
		return $arrRet;
	}

	function fetch_structure(&$msg, &$email) {
		$email = array();
		
		$separador = "\r\n\r\n";
		$bodypos = strpos($msg, $separador);
		if ($bodypos !== false) {
			$email['header'] = trim(substr($msg, 0, $bodypos));
			$email['body'] = substr($msg, $bodypos+strlen($separador));

			$pos = strpos($email['header'], "\r\n");
			if ($pos != false)
				$line = substr($email['header'], 0, $pos);
			else
				$line = &$email['header'];

			if (strpos($line, ':') === false && strpos($line, ' ') === false) {
				$email['header'] = '';
				$email['body'] = trim($msg);
			}
		}
		else {
			$pos = strpos($msg, "\r\n");
			if ($pos != false)
				$line = substr($msg, 0, $pos);
			else
				$line = &$msg;

			if (strpos($line, ':') === false && strpos($line, ' ') === false) {
				$email['header'] = '';
				$email['body'] = trim($msg);
			}
			else {
				$email['header'] = trim($msg);
				$email['body'] = '';
			}
		}
	}
	
	function get_quotastring(&$string) {
		$value = '';
		$ilen = strlen($string);
		$start = 0;
		$quota = false;
		
		if ($string[0] == '"') {
			$start = 1;
			$quota = true;
		}
		
		$i = $start;
		while ($i < $ilen){
			if ($quota) {
				if ($string[$i] == '"')
					break;
			}
			else {
				if ($string[$i] == ';' || $string[$i] == ' ')
					break;
			}
			
			$i++;
		}
		
		return substr($string, $start, $i-$start);
	}

	function get_boundary(&$ctype){
		$boundary = '';
		
		if (preg_match('/boundary[ ]?=[ ]?(["]?.*)/i', $ctype, $regs)) {
			$boundary = '--'.$this->get_quotastring(trim($regs[1]));
		}
		
		return $boundary;
	}
	
	function get_filename(&$cdisp, &$ctype) {
		$filename = '';
		if (preg_match('/filename[*]?[0-9]?[*]?[ ]?=[ ]?(["]?.*)/i', $cdisp, $regs)) {
			$temp = trim($regs[0]);
			if (strncmp($temp, 'filename*', 9) == 0) {
				$arrtemp = explode(';', $temp);
				foreach($arrtemp as &$item) {
					$item = trim($item);
					if (strncmp($item, 'filename*', 9) == 0) {
						$pos = strpos($item, '=');
						if ($pos !== false)
							$item = substr($item, $pos+1);
							
						$arr = explode('\'\'', $item);
						if ($arr[1] != '')
							$item = $arr[1];
						
						$item = trim($item);
						$item = str_replace('"', '', $item);
						if ($item != '')
							$filename .= $item;
					}
				}

				if ($filename != '' && strpos($filename, '=?') === false)
					$filename = '=?'.$arr[0].'?B?'.base64_encode(rawurldecode($filename)).'?=';
			}
			else {
				$filename = $this->get_quotastring(trim($regs[1]));
			}
		}

		if ($filename == '') {
			if (preg_match('/name[*]?[0-9]?[*]?[ ]?=[ ]?(["]?.*)/i', $ctype, $regs)) {
				$temp = trim($regs[0]);
				if (strncmp($temp, 'name*', 5) == 0) {
					$arrtemp = explode(';', $temp);
					foreach($arrtemp as &$item) {
						$item = trim($item);
						if (strncmp($item, 'name*', 5) == 0) {
							$pos = strpos($item, '=');
							if ($pos !== false)
								$item = substr($item, $pos+1);

							$arr = explode('\'\'', $item);
							if ($arr[1] != '')
								$item = $arr[1];
								
							$item = trim($item);
							$item = str_replace('"', '', $item);
							if ($item != '')
								$filename .= $item;
						}
					}
					
					if ($filename != '' && strpos($filename, '=?') === false)
						$filename = '=?'.$arr[0].'?B?'.base64_encode(rawurldecode($filename)).'?=';
				}
				else {
					$filename = $this->get_quotastring(trim($regs[1]));
				}
			}
		}
		
		return $filename;
	}
	
	function get_charset($ctype) {
		$charset = '';

		if (preg_match('/charset[ ]?=[ ]?(["]?.*)/i', $ctype, $regs)) {
			$charset = $this->get_quotastring(trim($regs[1]));
		}
		
		return $charset;
	}
	
	function detect_charset($str) {
		global $default_charset;
		
		if (check_utf8($str))
			return 'utf-8';
		
		$charset = mb_detect_encoding($str, 'ASCII,UTF-8,'.$default_charset.',GB18030,EUC-CN,EUC-JP,EUC-KR');
		
		return $charset;
	}

	function make_link_clickable(&$str){
		$str = preg_replace('/([[:space:]])((f|ht)tp[s]?:\/\/[a-z0-9~#%@\&:;=?+\/\.,_-]+[a-z0-9~#%@\&=?+\/_.;-]+)/i', '\\1<a href="\\2" target="_blank">\\2</a>', $str); //http 
		$str = preg_replace('/([[:space:]])(www\.[a-z0-9~#%@\&:;=?+\/\.,_-]+[a-z0-9~#%@\&=?+\/_.;-]+)/i', '\\1<a href="http://\\2" target="_blank">\\2</a>', $str); // www. 
		$str = preg_replace('/([[:space:]])([_\.0-9a-z=-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{1,4})/i', '\\1<a href="mailto:\\2">\\2</a>', $str); // mail 
		$str = preg_replace('/([[:space:]])(&lt;)([_\.0-9a-z=-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{1,4})(&gt;)/i', '\\1\\2<a href="mailto:\\3">\\3</a>\\5', $str); // mail 

		$str = preg_replace('/^((f|ht)tp[s]?:\/\/[a-z0-9~#%@\&:;=?+\/\.,_-]+[a-z0-9~#%@\&=?+\/_.;-]+)/i', '<a href="\\1" target="_blank">\\1</a>', $str); //http 
		$str = preg_replace('/^(www\.[a-z0-9~#%@\&:;=?+\/\.,_-]+[a-z0-9~#%@\&=?+\/_.;-]+)/i', '<a href="http://\\1" target="_blank">\\1</a>', $str); // www. 
		$str = preg_replace('/^([_\.0-9a-z=-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{1,4})/i', '<a href="mailto:\\1">\\1</a>', $str); // mail 
		$str = preg_replace('/^(&lt;)([_\.0-9a-z=-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{1,4})(&gt;)/i', '\\1<a href="mailto:\\2">\\2</a>\\4', $str); // mail 

		return $str;
	}

	function filter_scripts(&$data) {
		if ($this->allow_scripts)
			return; 
		
		if (empty($data)) 
			return; 

		if (strlen($data) > 5120)  // > 5K not filter 
			return; 

		$newdata = HTMLFilter($data, 'images/blank.gif', $this->block_external_images);
		$newdata = trim($newdata);
		
		//if (!empty($newdata)) 
			$data = $newdata;
	}

	function html2text(&$str) {
		return my_strip_tags($str);
	}

	function remove_crlfdot(&$msg) {
		$msg = str_replace("\r\n..", "\r\n.", $msg);
		$msg = trim($msg, "\r\n.");
	}

	/*
	Decode UUEncoded attachments
	*/
	function uudecode(&$data) {
		$b64chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/A';
		$uudchars = '`!"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_ ';
		$lines = preg_split("/\r?\n/", $data);
		$encode = '';
		foreach ($lines as &$line) {
			if ($line != '') {
				$count   = (ord($line[0])-32) % 64;
				$count   = ceil(($count*4) / 3);
				$encode .= substr(ltrim($line), 1, $count);
			}
		}
		$encode = strtr($encode, $uudchars, $b64chars);
		
		$len = strlen($encode);
		while($len % 4) {
			$encode .= '=';
			$len++;
		}

		return base64_decode($encode);
	}

	/*
	Guess all UUEncoded in the body
	*/
	function extract_uuencoded(&$body) {
		$regex = "/(begin ([0-7]{3}) (.+))\r\n(.+)\r\nend/Us";
        preg_match_all($regex, $body, $matches);
        
		$index = is_array($this->_content['attachments']) ? count($this->_content['attachments']) : 0;

        $matchecount = count($matches[3]);
        for ($i = 0; $i < $matchecount; $i++) {
			$boundary	= $matches[1][$i];
			$fileperm	= $matches[2][$i];
			$filename	= $matches[3][$i];
			$stream		= $this->uudecode($matches[4][$i]);

			if (strpos($filename, "\0"))
				$filename = str_replace("\0", '', $filename);

			$arrTemp = Array();
			$arrTemp['index']				= $index++;
			$arrTemp['name']['value'] 		= $filename;
			$arrTemp['name']['charset']		= '';
			$arrTemp['size'] 				= strlen($stream);
			$arrTemp['content-type'] 		= 'application/unknown';
			$arrTemp['content-disposition'] = 'attachment';
			$arrTemp['filename']			= $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_userid.'_'.$this->msg_sessid.'_'.md5($this->msg_folder.'_'.$this->msg_id.'_'.join('|', array_merge($this->current_level, array($arrTemp['index']))));
			$arrTemp['embed'] 				= false;
			
			$this->_content['attachments'][] = $arrTemp;
			file_put_contents($arrTemp['filename'], $stream);
			
			unset($stream);
			unset($arrTemp);
		}
		
		$body = preg_replace($regex, '', $body);
		
		return $matchecount;
	}

	/*
	Extract all attachmentes contained in a MS-TNEF attachment
	*/
	function extract_tnef(&$body, $boundary) {
		$added = 0;
		$objcount = 0;
		$index = is_array($this->_content['attachments']) ? count($this->_content['attachments']) : 0;
		
		if ($this->decode_mstnef) {
			$tnef = new TNEF();
			$tnefobj = $tnef->Decode($body);
	
			$objcount = is_array($tnefobj) ? count($tnefobj) : 0;
		}

		if ($objcount != 0) {
			for($i = 0; $i < $objcount; $i++) {
				$tnefobj[$i]['name'] = str_replace("\0", '', $tnefobj[$i]['name']);
					
				if ($tnefobj[$i]['msgbody'] == 1) {
					unset($this->_content['body']);
					
					$ctype = '';
					if ($tnefobj[$i]['charset'] != '')
						$ctype = 'charset=\"'.$tnefobj[$i]['charset'].'"';
						
					$this->add_body($tnefobj[$i]['stream'], $ctype);
					
					continue;
				}
				
				$arrTemp = Array();
				$arrTemp['index']				= $index++;
				$arrTemp['name']['value'] 		= $tnefobj[$i]['name'];
				$arrTemp['name']['charset']		= $tnefobj[$i]['charset'];
				$arrTemp['size'] 				= $tnefobj[$i]['size'];
				$arrTemp['content-type'] 		= $tnefobj[$i]['type0'].'/'.$tnefobj[$i]['type1'];
				$arrTemp['content-disposition'] = 'attachment';
				$arrTemp['filename'] 			= $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_sessid.'_'.md5(uniqid(microtime(), true));
				$arrTemp['embed'] 				= false;
				if ($tnefobj[$i]['content-id']) {
					$tnefobj[$i]['content-id'] = str_replace("\0", '', $tnefobj[$i]['content-id']);
					$arrTemp['content-id'] 		= $tnefobj[$i]['content-id'];
				}
				
				$this->_content['attachments'][] = $arrTemp;
				file_put_contents($arrTemp['filename'], $tnefobj[$i]['stream']);
				
				$added++;
				
				unset($content);
				unset($arrTemp);
			}
		}
		
		if ($added == 0) {
			$arrTemp = Array();
			$arrTemp['index']				= $index++;
			$arrTemp['name']['value'] 		= 'winmail.dat';
			$arrTemp['name']['charset']		= '';
			$arrTemp['size'] 				= strlen($body);
			$arrTemp['content-type'] 		= 'application/ms-tnef';
			$arrTemp['content-disposition'] = 'attachment';
			$arrTemp['filename'] 			= $this->temp_folder.'_attachments'.DIRECTORY_SEPARATOR.$this->msg_sessid.'_'.md5(uniqid(microtime(), true));
			$arrTemp['embed'] 				= false;
			
			$this->_content['attachments'][] = $arrTemp;
			file_put_contents($arrTemp['filename'], $body);
				
			unset($body);
			unset($arrTemp);
		}

		unset($tnef);
		
		return $objcount;
	}
}

}
?>